use x86_64::structures::idt::{InterruptStackFrame, PageFaultErrorCode};
use crate::{events::EVENTMAN, interrupts::InterruptIndex};

mod ps2_keyboard;

pub use ps2_keyboard::ps2_kb_int;

pub extern "x86-interrupt" fn breakpoint(sf: InterruptStackFrame){
    log::error!("EXCEPT: BREAKPOINT\n{:#?}", sf);
}

pub extern "x86-interrupt" fn timer_interrupt(_: InterruptStackFrame){
    //log::debug!("INT: TIMER\n{:#?}", sf);
    // print!(".");
    EVENTMAN.lock().dispatch(&crate::events::Event::TimerInterrupt(None));
    unsafe {
        super::PICS.lock()
            .notify_end_of_interrupt(InterruptIndex::Timer as u8);
    }
}
pub extern "x86-interrupt" fn double_fault(stack_frame: InterruptStackFrame, _: u64) -> ! {
    panic!("EXCEPT: DOUBLE FAULT \n{:#?}", stack_frame);
}

pub extern "x86-interrupt" fn page_fault(stack_frame: InterruptStackFrame, error_code: PageFaultErrorCode) {
    use x86_64::registers::control::Cr2;

    log::error!("EXCEPT: PAGE FAULT");
    log::debug!("Addr:  {:?}", Cr2::read());
    log::debug!("ErrCo: {:?}\n{:#?}", error_code, stack_frame);
    crate::utils::hcf();
}
