
use log::{Level, LevelFilter, Metadata, Record, SetLoggerError};

use crate::logger::serial::{write_com, ANSI_COLORS, COM0};

use self::vga::{Color, ColorCode, WRITER};

#[macro_use]
pub mod vga;
pub mod serial;
#[macro_use]
pub mod macros;

pub static LOGGER: KLogger = KLogger;

pub struct KLogger;



const C_TRACE: ColorCode = ColorCode::new(Color::Cyan, Color::Black);
const C_DEBUG: ColorCode = ColorCode::new(Color::Blue, Color::Black);
const C_INFO: ColorCode = ColorCode::new(Color::Green, Color::Black);
const C_WARN: ColorCode = ColorCode::new(Color::Yellow, Color::Black);
const C_ERROR: ColorCode = ColorCode::new(Color::Red, Color::Black);
const C_NORMAL: ColorCode = ColorCode::new(Color::White, Color::Black);


impl log::Log for KLogger {
    fn enabled(&self, _: &Metadata) -> bool {
        true
    }

    fn log(&self, record: &Record) {
        if self.enabled(record.metadata()) {
            {WRITER.lock().set_color(Color::Magenta, Color::Black); }
            print!("[");
            write_com(&COM0, format_args!("{}[", ANSI_COLORS::MAGENTA));
            match record.level() {
                Level::Trace => {
                    {WRITER.lock().set_color_code(C_TRACE); }
                    print!("TRACE");
                    write_com(&COM0, format_args!("{}TRACE", ANSI_COLORS::BLUE2));
                }
                Level::Debug => {
                    {WRITER.lock().set_color_code(C_DEBUG); }
                    print!("DEBUG");
                    write_com(&COM0, format_args!("{}DEBUG", ANSI_COLORS::BLUE));
                }
                Level::Info => {
                    {WRITER.lock().set_color_code(C_INFO); }
                    print!("INFO");
                    write_com(&COM0, format_args!("{}INFO", ANSI_COLORS::GREEN));
                }
                Level::Warn => {
                    {WRITER.lock().set_color_code(C_WARN); }
                    print!("WARN");
                    write_com(&COM0, format_args!("{}WARN", ANSI_COLORS::YELLOW));
                }
                Level::Error => {
                    {WRITER.lock().set_color_code(C_ERROR); }
                    print!("ERROR");
                    write_com(&COM0, format_args!("{}ERROR", ANSI_COLORS::RED));
                }
            }
            {WRITER.lock().set_color(Color::Magenta, Color::Black); }
            print!("]");
            {WRITER.lock().set_color_code(C_NORMAL); }

            write_com(&COM0, format_args!("{}]", ANSI_COLORS::MAGENTA));
            
            println!(" {}: {}", record.module_path().unwrap_or(""), record.args());

            write_com(&COM0, format_args!(" {}{}: {}\n", ANSI_COLORS::RESET, record.module_path().unwrap_or(""), record.args()));
        }
    }

    fn flush(&self) {}
}

pub fn init(lvl: LevelFilter) ->  Result<(), SetLoggerError> {
    log::set_logger(&LOGGER)?;
    log::set_max_level(lvl);
    log::info!("Logger init OK");

    Ok(())
}

